@@ -24,7 +24,7 @@
  * @author Samuel Mimram, David Baelde
  */
 
-/* $Id: mad_stubs.c 2694 2006-10-16 23:17:25Z dbaelde $ */
+/* $Id: mad_stubs.c 2919 2007-03-02 14:25:16Z metamorph68 $ */
 
 #include <caml/alloc.h>
 #include <caml/callback.h>
@@ -149,7 +149,11 @@ CAMLprim value ocaml_mad_openfile(value file)
     caml_raise_with_arg(*caml_named_value("mad_exn_openfile_error"), caml_copy_string(strerror(errno)));
   mf->buf = (unsigned char*)malloc(BUFFER_SIZE);
 
-  block = caml_alloc_custom(&madfile_ops, sizeof(madfile_t*), 0, 1);
+  // The amount of data "secretly" owned by the madfile_t may be underestimated
+  // here since I only take into account mf->buf, while it may be possible
+  // that the mad_*_init functions also allocate stuff.
+  block = caml_alloc_custom(&madfile_ops, sizeof(madfile_t*),
+                            BUFFER_SIZE, 1000000);
   Madfile_val(block) = mf;
 
   CAMLreturn(block);
@@ -254,7 +258,7 @@ CAMLprim value ocaml_mad_decode_frame(value madf)
       if (read_size>0)
         memcpy(read_start, String_val(Field(data, 0)), read_size);
       else
-        if (read_size=0)
+        if (read_size==0)
           caml_raise_constant(*caml_named_value("mad_exn_end_of_stream"));
         else
           caml_raise_with_arg(*caml_named_value("mad_exn_read_error"),
@@ -343,7 +347,7 @@ CAMLprim value ocaml_mad_decode_frame(value madf)
    * skip the faulty part and re-sync to the next frame.
    */
   dec = mad_frame_decode(&mf->frame, &mf->stream);
-  if (dec)
+  if (dec) {
     if (MAD_RECOVERABLE(mf->stream.error)
         || mf->stream.error == MAD_ERROR_BUFLEN) {
       caml_leave_blocking_section();
@@ -352,6 +356,7 @@ CAMLprim value ocaml_mad_decode_frame(value madf)
       caml_leave_blocking_section();
       caml_raise_with_arg(*caml_named_value("mad_exn_mad_error"), caml_copy_string(mad_stream_errorstr(&mf->stream)));
     }
+  }
 
   /* Accounting. The computed frame duration is in the frame
    * header structure. It is expressed as a fixed point number
